package lulla

import scala.math.pow
import better.files._

object Run extends App {
  println(
    Lulla.run(
      file = File("acquisition10_1.csv").toJava,
      beta0 = 0.61815,
      xi = 150.0,
      p0 = 1.1955e-4,
      pi = -0.000973,
      state = Vector(80000.0, 20000.0, 0.0)
    )
    //.map(_.toVector).mkString("\n")
  )
}


object Lulla {

  val lambda = 0.33
  val N = 1e5

  def run(file: java.io.File, beta0: Double, xi: Double, p0: Double, pi: Double, state: Vector[Double], integrationStep: Double = 0.01) = {
    val columns = File(file.getAbsolutePath).lines.map(_.split(",")).toVector
    val ft = columns.map(_(4).toDouble / N)

    val steps = Lulla.dynamic(beta0, xi, p0, pi, ft, state, integrationStep, (0 until ft.size).map(_.toDouble).toVector)
    
    val Is = state(2) :: steps.map(_(2)).sliding(2).map { case Vector(t1, t2) => t2 - t1 }.toList
    val infections = columns.map(_(6).toDouble).toList

    val likelihood =
      (Is zip infections).map { case(i, inf) =>
        if(i > 0) (inf * math.log(i) - i) else 0
      }.sum

    val portageMean = steps.map(_(1)).sum / steps.size
    val llPortage = 0.3 * N * math.log(portageMean) - portageMean

    (likelihood + llPortage) match {
      case Double.NegativeInfinity | Double.NaN => Double.PositiveInfinity
      case x => -x
    }
  }



  def dynamic(beta0: Double, xi: Double, p0: Double, pi: Double, f: Vector[Double], state: Vector[Double], integrationStep: Double, timeSteps: Vector[Double])= {
    def ft(t: Double) = f(math.floor(t).toInt)
    def betaT(t: Double) = beta0 * (1 + xi * ft(t))
    def pT(t: Double) = p0 * (1 + pi * ft(t))


    def sDot(state: Array[Double], t: Double) =
      (lambda * state(1)) - (betaT(t) * state(1) * state(0)) / N

    def cDot(state: Array[Double], t: Double) =
      -(lambda * state(1)) + (betaT(t) * state(1) * state(0)) / N

    def iDot(state: Array[Double], t: Double) =
      pT(t) * state(1)

    val dynamic = Dynamic(sDot, cDot, iDot)
    dynamic.integrate(state.toArray, integrationStep, timeSteps)
  }

}
